----------------------------------------------------------------------------------
-- Summer school on real-world crypto and privacy - Sibenik, Croatia, 2016
-- Authors: Ricardo Chaves & Nele Mentens
-- 
-- Create Date:    05/31/2016 
-- Design Name:    Noekeon
-- Module Name:    NoekeonRoundCommonLoop
-- Target Devices: Xilinx
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity NoekeonRoundCommonLoop is
    Port ( a_in : in  STD_LOGIC_VECTOR (127 downto 0);
           key : in  STD_LOGIC_VECTOR (127 downto 0);
           round : in  STD_LOGIC_VECTOR (3 downto 0);
           reset : in  STD_LOGIC;
           selectInput : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           a_out : out  STD_LOGIC_VECTOR (127 downto 0));
end NoekeonRoundCommonLoop;

architecture Behavioral of NoekeonRoundCommonLoop is

	COMPONENT NoekeonRound
	PORT(
		a_in : IN std_logic_vector(127 downto 0);
		key : IN std_logic_vector(127 downto 0);
		round : IN std_logic_vector(3 downto 0);
		a_out : OUT std_logic_vector(127 downto 0)
		);
	END COMPONENT;


   COMPONENT Theta
	PORT(
		a_in : IN std_logic_vector(127 downto 0);
		key : IN std_logic_vector(127 downto 0);          
		a_out : OUT std_logic_vector(127 downto 0)
		);
	END COMPONENT;
   
	COMPONENT Reg128
	PORT( 
      data_in : in  STD_LOGIC_VECTOR (127 downto 0);
      reset : in  STD_LOGIC;
      clk : in  STD_LOGIC;
      data_out : out  STD_LOGIC_VECTOR (127 downto 0)
		);
	END COMPONENT;

signal a, a_midle, a_round, a_reg, a_in_tlast,a_out_tlast: STD_LOGIC_VECTOR(127 downto 0);
signal RC1_last, RC2cst: STD_LOGIC_VECTOR(7 downto 0);
signal round2: STD_LOGIC_VECTOR(3 downto 0);

begin

--  for(i=0 ; i<NROUND ; i++)
a  <= a_reg when selectInput = '0' else a_in;
 
Inst_NoekeonRound: NoekeonRound PORT MAP(
		a_in => a,
		key => key,
		round => round,
		a_out => a_midle
	);

round2 <= round OR "0001";

Inst_NoekeonRound2: NoekeonRound PORT MAP(
            a_in => a_midle,
            key => key,
            round => round2,
            a_out => a_round
        );

--
 
Inst_Reg128: Reg128 PORT MAP(
		data_in => a_round,
		reset => reset,
        clk => clk,
		data_out => a_reg
	);
 
--------- End of loop -------------
  
--  a0^=RC1;
RC1_last <= X"D4";

a_in_tlast(32*3-1 downto 0) <= a_round(32*3-1 downto 0);
a_in_tlast(32*3 +31 downto 32*3+8) <= a_round(32*3 +31 downto 32*3+8);  

a_in_tlast(32*3 +7 downto 32*3) <= a_round(32*3 +7 downto 32*3) xor RC1_Last;

--  Theta(k,a);    
Inst2_Theta: Theta PORT MAP(
		a_in => a_in_tlast,
		key  => key,
		a_out => a_out_tlast
	   );

--  a0^=RC2; 
RC2cst <= X"00";

a_out(32*3-1 downto 0) <= a_out_tlast(32*3-1 downto 0);
a_out(32*3 +31 downto 32*3+8) <= a_out_tlast(32*3 +31 downto 32*3+8);  

a_out(32*3 +7 downto 32*3) <= a_out_tlast(32*3 +7 downto 32*3) xor RC2cst;

 

end Behavioral;

